;(function($){
    var container,canvas,ratioScale,args;
    $.fn.drawPieChart = function(options) {
        args = $.extend({}, $.fn.drawPieChart.defaults, options);
        return this.each(function() {
            container = this;
            creatCanvas(); //创建画板
            varInit(); //组件变量初始化
        });
    };
    // 默认参数
    $.fn.drawPieChart.defaults = {
        radius : 150,
        startPoint : 0,
        anticlockwise : false,
        color : ["#7edae2","#7eb5e2","#a3a3e5","#9b9b9b","#cccccc","#c3c3e2","#cccccc"],
        segment : true,
        textDis: 70, //文本框宽度
        textSize : 14,
        lineHeight : 1, //文本行距 
    };
    // 开放接口方法
    $.drawPieChart = ({
        firstDraw: function(canv){ //初次绘制
            firstDrawPie(canv);
        },
    });
    //创建画板
    function creatCanvas(){
        ratioScale = window.devicePixelRatio;
        var width = $(container).width()*ratioScale;
        var height = (args.radius + args.textDis) * 2 * ratioScale;
        height = height<width ? height : width;
        var htmlText = '<canvas width="'+ width +'" height="'+ height +'" style="width:100%;"></canvas><div class="info"><span></span></div>';
        $(container).append(htmlText);
        canvas = $(container).find('canvas')[0];
        canvas.centerP = {};
        canvas.centerP.x = Math.floor(width/2);
        canvas.centerP.y = Math.floor(height/2);
    }
    //组件变量初始化
    function varInit(){
        canvas.data = args.data.sort(dataSortFun("rate")); //从大到小排序
        canvas.totalNum = getTotalNum(); //获取总数
        canvas.ctx = canvas.getContext("2d");
        canvas.startPoint = args.startPoint / 180 * Math.PI;
        canvas.color = args.color;
        canvas.radius = canvas.centerP.y - args.textDis * ratioScale; //半径
        canvas.textRadius = canvas.centerP.y - args.textDis/2 * ratioScale; //文本半径
        canvas.dire = args.anticlockwise; //顺时针逆时针
        canvas.segment = args.segment; //色块间是否加区隔
        canvas.neg = 1; //顺时针
        if(canvas.dire){//逆时针
            canvas.neg = -1;
        }
        canvas.textSize = args.textSize * ratioScale; //标注文字字号
        canvas.ctx.font = canvas.textSize +"px 'calibri','STHeiti'";
        if(isEng(canvas.data[0].key)){
            canvas.ctx.font = Math.floor(canvas.textSize*1.3) +"px 'calibri','STHeiti'";
            args.lineHeight *= 1.2;
        }
        canvas.ctx.textBaseline = "middle";
    }
    //排序条件函数
    function dataSortFun(key){
        return function(obj1,obj2){
            var val1 = obj1[key];
            var val2 = obj2[key];
            return val2-val1;
        }
    }
    //获取总数
    function getTotalNum(){
        var total = 0;
        for(var i=0,len=canvas.data.length; i<len; i++){
            total += parseInt(canvas.data[i].rate);
        }
        return total;
    }

    //绘制饼图
    function drawPie(canvas,aniPro,clickId){
        canvas.ctx.clearRect(0,0,canvas.width,canvas.height);
        var totalOffset = 0; //Y轴累积偏移
        var startPoint = canvas.startPoint;
        var lastRectHubP={},lastRectLen,lastRectHei;
        for(var i=0,len=canvas.data.length; i<len; i++){
            canvas.ctx.fillStyle = canvas.color[i];            
            canvas.ctx.beginPath();
            canvas.ctx.moveTo(canvas.centerP.x,canvas.centerP.y);
            var offsetAngle = Math.PI*2*(canvas.data[i].rate/canvas.totalNum)*canvas.neg*aniPro;
            var endPoint = startPoint + offsetAngle;
            var textAngle = startPoint + offsetAngle/2;
            var drawRadius = canvas.radius;
            if(i == clickId){
                drawRadius *= 1.05;
            }
            canvas.ctx.arc(canvas.centerP.x,canvas.centerP.y,drawRadius,startPoint,endPoint,canvas.dire);
            canvas.ctx.fill();
            if(canvas.segment){
                canvas.ctx.strokeStyle = "#fff";
                canvas.ctx.stroke();
            }
            startPoint = endPoint;
            //绘制说明文本
            if(aniPro > 0.8){
                canvas.ctx.beginPath();
                var textP = {};
                textP.x = Math.floor(canvas.centerP.x + Math.cos(textAngle)*canvas.textRadius);
                textP.y = Math.floor(canvas.centerP.y + Math.sin(textAngle)*canvas.textRadius) + totalOffset;
                var lineP = {};
                lineP.x = Math.floor(canvas.centerP.x + Math.cos(textAngle)*canvas.radius);
                lineP.y = Math.floor(canvas.centerP.y + Math.sin(textAngle)*canvas.radius);
                var textArr = spText(canvas.data[i].key); //分割字符串，返回数组
                var length = 0; //文本块宽度（px）
                for(var j = 0,txtlen = textArr.length; j < txtlen; j++){
                    var newLength = canvas.ctx.measureText(textArr[j]).width;
                    if(newLength > length){
                        length = newLength;
                    }
                }
                var height = canvas.textSize * txtlen; //文本块高度（px）
                var rectHubP = {}; //文本块中心点坐标
                //文字坐标修正
                if(textP.x < canvas.centerP.x){
                    canvas.ctx.textAlign = "right";
                    if(textP.x - length < 0){
                        textP.x = length;
                    }
                    rectHubP.x = textP.x - length/2;
                }
                else {
                    canvas.ctx.textAlign = "left";
                    if(textP.x + length > canvas.width){
                        textP.x = canvas.width - length;
                    }
                    rectHubP.x = textP.x + length/2;
                }            
                rectHubP.y = textP.y - canvas.textSize/2 + height/2;
                
                var offsetY = 0;
                if(lastRectLen){
                    offsetY = coordConflict(rectHubP,length+5,height+5,lastRectHubP,lastRectLen,lastRectHei,canvas.centerP); //与上一个文字的重叠检测,返回Y轴重叠高度
                }
                if(offsetY){
                    textP.y += offsetY * canvas.neg;
                    rectHubP.y = textP.y - canvas.textSize/2 + height/2;
                    totalOffset += offsetY * canvas.neg;
                }
                //保存文本块位置，为下次检测做准备
                lastRectHubP = rectHubP;
                lastRectLen = length+5;
                lastRectHei = height+5;
                //绘制文本
                canvas.ctx.fillStyle = "#4a4a4a";
                for(j = 0; j < txtlen; j++){
                    canvas.ctx.fillText(textArr[j], textP.x, textP.y+canvas.textSize*j*args.lineHeight);
                }
                canvas.ctx.strokeStyle = "#9a9a9a"; //连线颜色
                canvas.ctx.moveTo(lineP.x, lineP.y);
                canvas.ctx.lineTo(textP.x, textP.y);
                canvas.ctx.stroke();
            }
        }        
    }

    //判断英文字符
    function isEng(str){
        var re = /[0-9a-zA-Z-]/;
        if(re.test(str)) return true;
        return false;
    }
    //判断中文标点
    function isCPun(str){
        var re = /[，。？：；‘’！“”—……、]|(－{2})|(（）)|(【】)|({})|(《》)/;
        if(re.test(str)) return true;
        return false;
    }
    //判断标点
    function isPun(str){
        var re =  /([\[\]\,.?"\(\)+_*\/\\&\$#^@!%~`<>:;\{\}？，。·！￥……（）+｛｝【】、|《》]|(?!\s)'\s+|\s+'(?!\s))/ig;
        if(re.test(str)) return true;
        return false;
    }
    //分割字符串，返回数组
    function spText(txt){
        var step = Math.floor(args.textDis / args.textSize);
        var txtLen = txt.length;
        var txtArr = [];
        if(txtLen < step){
            txtArr.push(txt);
            return txtArr;
        }
        for(var start = 0; start < txtLen;){
            var tep = step;
            for(var i=0;i<tep;i++){
                if(isEng(txt[start+i])){
                    tep += 0.5;
                }
            }
            tep = Math.floor(tep);
            if(isPun(txt[start+tep])){
                tep--;
            }
            txtArr.push(txt.substr(start,tep));
            start += tep;
        }
        return txtArr;
    }
    //与上一个文字的重叠检测,返回Y轴重叠高度
    function coordConflict(rectHubP,length,height,lastRectHubP,lastRectLen,lastRectHei,centerP){
        if((Math.abs(rectHubP.x - lastRectHubP.x) < (length + lastRectLen)/2) && (Math.abs(rectHubP.y - lastRectHubP.y) < (height + lastRectHei)/2)){
            if(rectHubP.x < centerP.x){
                return (height + lastRectHei)/2 + (rectHubP.y - lastRectHubP.y);
            }
            return (height + lastRectHei)/2 - (rectHubP.y - lastRectHubP.y);
        }
        return 0;
    }
    //绑定鼠标点击事件
    function blindClickEvent(canv){
        $(canv).on('click',function(e){
            var clickP = getPointOnCanvas(this,e.clientX, e.clientY);
            if(Math.pow(clickP.x - this.centerP.x,2) + Math.pow(clickP.y - this.centerP.y,2) < Math.pow(this.radius,2)){
                var radina = getRadina(clickP.x,clickP.y,this.centerP.x,this.centerP.y);
                var startPoint = this.startPoint, clickId = -1;
                if(this.dire){//逆时针
                    var offsetRadina = -Math.PI*2 + radina + startPoint;
                    for(var i =0, len = this.data.length; i < len; i++){
                        var endPoint = startPoint - Math.PI*2*(this.data[i].rate/this.totalNum);
                        if(offsetRadina > endPoint){
                            clickId = i;
                            break;
                        }
                        startPoint = endPoint;
                    }
                }
                else {//顺时针
                    var offsetRadina = radina - startPoint;
                    for(var i =0, len = this.data.length; i < len; i++){
                        var endPoint = startPoint + Math.PI*2*(this.data[i].rate/this.totalNum);
                        if(offsetRadina < endPoint){
                            clickId = i;
                            break;
                        }
                        startPoint = endPoint;
                    }
                }
                //重绘饼图
                if(clickId != this.lastClickId){
                    drawPie(this, 1, clickId);
                    this.lastClickId = clickId;
                    var info = $(this).next();
                    var percent = Math.floor(this.data[clickId].rate/this.totalNum*100);
                    if(!percent) {
                        percent = 1;
                    }
                    info.find("span").html(this.data[clickId].key +"："+percent+"%");
                    info.addClass('show');
                }
                else {
                    drawPie(this,1);
                    this.lastClickId = null;
                    $(this).next().removeClass('show');
                }
            }
            else {
                drawPie(this,1);
                this.lastClickId = null;
                $(this).next().removeClass('show');
            }
        });
    }
    //得到鼠标点击在canvas里对应的坐标
    function getPointOnCanvas(canvas, x, y){
        var bbox =canvas.getBoundingClientRect();
        return { x: (x- bbox.left) * ratioScale,
                y: (y - bbox.top) * ratioScale};
    }
    //获取弧度
    function getRadina(x1, y1, x2, y2){
        // 直角的边长
        var x = Math.abs(x1 - x2);
        var y = Math.abs(y1 - y2);
        // 斜边长
        var z = Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
        // 余弦
        var cos = x / z;
        // 弧度
        var radina = Math.acos(cos);
        if(x1<x2 && y1>y2){ //第三象限
            radina = Math.PI - radina;
        }
        else if(x1<x2 && y1<y2){ //第四象限
            radina += Math.PI;
        }
        else if(x1>x2 && y1<y2){ //第一象限
            radina = Math.PI*2 - radina;
        }
        return radina;
    }

    //初始绘制饼图
    function firstDrawPie(canv){
        if(!window.requestAnimationFrame){
            var lastTime = 0;
            window.requestAnimationFrame = function(callback, element) {
                var currTime = new Date().getTime();
                var timeToCall = Math.max(0, 16 - (currTime - lastTime));
                var id = window.setTimeout(function() {
                    callback(currTime + timeToCall);
                }, timeToCall);
                lastTime = currTime + timeToCall;
                return id;
            };
        }
        var pro = 1;
        recuDrawPie(pro,canv);
    }
    //初次绘制饼图递归函数
    function recuDrawPie(pro,canv){
        pro /= 1.08;
        if(pro > 0.001){
            drawPie(canv, 1-pro);
            // setTimeout(function(){
            //     recuDrawPie(pro,canv);
            // },40);
            requestAnimationFrame(function(){
                recuDrawPie(pro,canv);
            });
        }
        else {
            drawPie(canv, 1);
            blindClickEvent(canv); //绑定点击事件
        }
    }

})(Zepto);